@@ -1,19 +1,25 @@
-import numpy as np
-import pandas as pd
 import copy
 import json
-import matplotlib.pyplot as plt
+import numpy as np
+import pandas as pd
 import seaborn as sns
-from category_encoders import CatBoostEncoder
+import matplotlib.pyplot as plt
+
+from category_encoders import CountEncoder
+
+from scipy.special import rel_entr
 from scipy.spatial.distance import jensenshannon
 from scipy.stats import gaussian_kde, ks_2samp, chisquare, wasserstein_distance
-from scipy.special import rel_entr
-from sklearn.model_selection import RandomizedSearchCV, train_test_split
-from sklearn.ensemble import RandomForestRegressor, RandomForestClassifier
-from sklearn.metrics import (r2_score, mean_absolute_error, precision_score,
-                             recall_score, accuracy_score, f1_score,
-                             roc_auc_score)
+
 from sklearn.utils import shuffle
+from sklearn.model_selection import RandomizedSearchCV
+from sklearn.ensemble import RandomForestRegressor, RandomForestClassifier
+from sklearn.metrics import (
+    r2_score, mean_absolute_error, mean_absolute_percentage_error, 
+    explained_variance_score, precision_score, recall_score, accuracy_score,
+    f1_score, roc_auc_score, confusion_matrix
+    )
+
 
 import logging
 logger = logging.getLogger()
@@ -53,11 +59,13 @@ class DataDriftDetector:
         A list of numeric columns in the dataset, will be determined by
         column types if not provided
     """
-    def __init__(self,
-                 df_prior,
-                 df_post,
-                 categorical_columns=None,
-                 numeric_columns=None):
+    def __init__(
+            self,
+            df_prior,
+            df_post,
+            categorical_columns=None,
+            numeric_columns=None
+            ):
         assert isinstance(df_prior, pd.DataFrame),\
             "df_prior should be a pandas dataframe"
         assert isinstance(df_post, pd.DataFrame),\
@@ -75,31 +83,20 @@ class DataDriftDetector:
         df_post_ = df_post.copy()
 
         if categorical_columns is None:
-            categorical_columns = (
-                [c for c in df_prior_.columns if
-                df_prior_.dtypes[c] == 'object']
-            )
-            logger.info(
-                "Identified categorical column(s): {}".format(
-                ", ".join(categorical_columns))
-            )
+            categorical_columns = [
+                c for c in df_prior_.columns if df_prior_.dtypes[c] == 'object'
+            ]
+            logger.info(f"Identified categorical column(s): {categorical_columns}")
 
-        df_prior_[categorical_columns] = (
-            df_prior_[categorical_columns].astype(str)
-        )
-        df_post_[categorical_columns] = (
-            df_post_[categorical_columns].astype(str)
-        )
+        df_prior_[categorical_columns] = df_prior_[categorical_columns].astype(str)
+        df_post_[categorical_columns] = df_post_[categorical_columns].astype(str)
 
         if numeric_columns is None:
             num_types = ['float64','float32','int32','int64','uint8']
-            numeric_columns = (
-                [c for c in df_prior_.columns if
-                 df_prior_.dtypes[c] in num_types]
-            )
-            logger.info("Identified numeric column(s): {}".format(
-                ", ".join(numeric_columns))
-            )
+            numeric_columns = [
+                c for c in df_prior_.columns if df_prior_.dtypes[c] in num_types
+            ]
+            logger.info(f"Identified numeric column(s): {numeric_columns}")
 
         df_prior_[numeric_columns] = df_prior_[numeric_columns].astype(float)
         df_post_[numeric_columns] = df_post_[numeric_columns].astype(float)
@@ -143,14 +140,15 @@ class DataDriftDetector:
             col_ = pd.concat([col_prior, col_post], ignore_index=True)
 
             # aggregate and convert to probability array
-            arr = (col_.groupby([col, '_source'])
-                       .size()
-                       .to_frame()
-                       .reset_index()
-                       .pivot(index=col, columns='_source')
-                       .droplevel(0, axis=1)
-                       .pipe(lambda df: df.loc[df.sum(axis=1).sort_values(ascending=False).index, :])
-                  )
+            arr = (
+                col_.groupby([col, '_source'])
+                    .size()
+                    .to_frame()
+                    .reset_index()
+                    .pivot(index=col, columns='_source')
+                    .droplevel(0, axis=1)
+                    .pipe(lambda df: df.loc[df.sum(axis=1).sort_values(ascending=False).index, :])
+                )
 
             arr_ = arr.div(arr.sum(axis=0),axis=1)
             arr_.fillna(0, inplace=True)
@@ -163,19 +161,14 @@ class DataDriftDetector:
                 rel_entr(arr_['prior'].to_numpy(), arr_['post'].to_numpy())
             )
 
-            jsd = jensenshannon(arr_['prior'].to_numpy(),
-                                arr_['post'].to_numpy())
-            wd = wasserstein_distance(arr_['prior'].to_numpy(),
-                                      arr_['post'].to_numpy())
+            jsd = jensenshannon(arr_['prior'].to_numpy(), arr_['post'].to_numpy())
+            wd = wasserstein_distance(arr_['prior'].to_numpy(), arr_['post'].to_numpy())
 
-            # chisquare test requires at least 5 
-            arr = arr.loc[arr.sum(axis=1) >= 5,]
             arr['post'] = arr['post'] / arr['post'].sum() * arr['prior'].sum()
             arr.fillna(0, inplace=True)
             
             # calculate test of similarity
-            cs_test = chisquare(arr['post'].to_numpy(),
-                                arr['prior'].to_numpy())
+            cs_test = chisquare(arr['post'].to_numpy(), arr['prior'].to_numpy())
 
             cat_res.update({
                 col: {
@@ -185,7 +178,6 @@ class DataDriftDetector:
                     'kl_divergence_prior_given_post': kl_prior_post,
                     'jensen_shannon_distance': jsd,
                     'wasserstein_distance': wd
-                    
                 }
             })
 
@@ -221,20 +213,23 @@ class DataDriftDetector:
                     'ks_2sample_test_p_value': ks_test[1],
                     'jensen_shannon_distance': jsd,
                     'wasserstein_distance': wd
-                    
                 }
             })
 
-        return {'categorical': dict(cat_res),
-                'numerical': dict(num_res)}
+        return {
+            'categorical': dict(cat_res),
+            'numerical': dict(num_res)
+            }
 
 
-    def plot_categorical_to_numeric(self,
-                                    plot_categorical_columns=None,
-                                    plot_numeric_columns=None,
-                                    categorical_on_y_axis=True,
-                                    grid_kws={'height':5},
-                                    plot_kws={}):
+    def plot_categorical_to_numeric(
+            self,
+            plot_categorical_columns=None,
+            plot_numeric_columns=None,
+            categorical_on_y_axis=True,
+            grid_kws={'height':5},
+            plot_kws={}
+            ):
         """Plots charts to compare categorical to numeric columns pairwise.
         Plots a pairgrid violin plot of categorical columns to numeric
         columns, split and colored by the source of datasets
@@ -269,10 +264,10 @@ class DataDriftDetector:
         col_nunique = df_prior.nunique()
 
         if plot_categorical_columns is None:
-            plot_categorical_columns = (
-                [col for col in col_nunique.index if
-                 (col_nunique[col] <= 20) & (col in self.categorical_columns)]
-            )
+            plot_categorical_columns = [
+                col for col in col_nunique.index if
+                (col_nunique[col] <= 20) & (col in self.categorical_columns)
+            ]
 
         if plot_numeric_columns is None:
             plot_numeric_columns = self.numeric_columns
@@ -306,31 +301,37 @@ class DataDriftDetector:
             y_cols = plot_numeric_columns
             x_cols = plot_categorical_columns
 
-        g = sns.PairGrid(data=plot_df,
-                         x_vars=x_cols,
-                         y_vars=y_cols,
-                         hue='_source',
-                         hue_kws={'split':True},
-                         **grid_kws)
+        g = sns.PairGrid(
+            data=plot_df,
+            x_vars=x_cols,
+            y_vars=y_cols,
+            hue='_source',
+            hue_kws={'split':True},
+            **grid_kws
+        )
 
-        g.map(sns.violinplot,
-              hue=plot_df['_source'],
-              split=True,
-              **plot_kws)
+        g.map(
+            sns.violinplot,
+            hue=plot_df['_source'],
+            split=True,
+            **plot_kws
+        )
 
         g.add_legend()
 
         return g
 
 
-    def plot_numeric_to_numeric(self,
-                                kind='scatter',
-                                diag_kind='kde',
-                                plot_kws=None,
-                                grid_kws=None,
-                                diag_kws={'common_norm':False},
-                                plot_numeric_columns=None,
-                                **kwargs):
+    def plot_numeric_to_numeric(
+            self,
+            kind='scatter',
+            diag_kind='kde',
+            plot_kws=None,
+            grid_kws=None,
+            diag_kws={'common_norm':False},
+            plot_numeric_columns=None,
+            **kwargs
+            ):
         """Plots charts to compare numeric columns pairwise.
         Plots a pairplot (from seaborn) of numeric columns, with a distribution
         plot on the diagonal and a scatter plot for all other charts
@@ -367,19 +368,18 @@ class DataDriftDetector:
         plot_df = pd.concat([df_prior, df_post])
         plot_df.reset_index(drop=True, inplace=True)
 
-        logger.info(
-            "Plotting the following numeric column(s): {}".format(
-            ", ".join(plot_numeric_columns))
-        )
+        logger.info(f"Plotting the following numeric column(s): {plot_numeric_columns}")
 
-        g = sns.pairplot(data=plot_df,
-                         kind=kind,
-                         diag_kind=diag_kind,
-                         hue='_source',
-                         plot_kws=plot_kws,
-                         diag_kws=diag_kws,
-                         grid_kws=grid_kws,
-                         **kwargs)
+        g = sns.pairplot(
+            data=plot_df,
+            kind=kind,
+            diag_kind=diag_kind,
+            hue='_source',
+            plot_kws=plot_kws,
+            diag_kws=diag_kws,
+            grid_kws=grid_kws,
+            **kwargs
+        )
 
         return g
 
@@ -399,14 +399,13 @@ class DataDriftDetector:
 
         col_nunique = self.df_prior.nunique()
         if plot_categorical_columns is None:
-            plot_categorical_columns = (
-                [col for col in col_nunique.index if
-                 (col_nunique[col] <= 20) & (col in self.categorical_columns)]
-            )
+            plot_categorical_columns = [
+                col for col in col_nunique.index if
+                (col_nunique[col] <= 20) & (col in self.categorical_columns)
+            ]
 
         logger.info(
-            "Plotting the following categorical column(s): {}".format(
-            ", ".join(plot_categorical_columns))
+            f"Plotting the following categorical column(s): {plot_categorical_columns}"
         )
 
         fig, ax = plt.subplots(len(plot_categorical_columns), 1,
@@ -430,12 +429,14 @@ class DataDriftDetector:
             _p2['_source'] = 'Post'
             _p = pd.concat([_p1, _p2])
 
-            sns.barplot(x="index",
-                        y="Proportion",
-                        hue="_source",
-                        data=_p,
-                        ax=_ax,
-                        **kwargs)
+            sns.barplot(
+                x=_p.index,
+                y="Proportion",
+                hue="_source",
+                data=_p,
+                ax=_ax,
+                **kwargs
+            )
             _ax.legend(loc='upper right', title='_source')
             _ax.set_xlabel(col)
             _ax.tick_params(axis='x', labelrotation=90)
@@ -450,19 +451,17 @@ class DataDriftDetector:
         return np.sqrt(np.mean((predictions-targets)**2))
 
 
-    def compare_ml_efficacy(self,
-                            target_column,
-                            test_data=None,
-                            OHE_columns=None,
-                            high_cardinality_columns=None,
-                            OHE_columns_cutoff=5,
-                            random_state=None,
-                            train_size=0.7,
-                            cv=3,
-                            n_iter=5,
-                            param_grid={'n_estimators': [100, 200],
-                                        'max_samples': [0.6, 0.8, 1],
-                                        'max_depth': [3, 4, 5]}):
+    def compare_ml_efficacy(
+            self,
+            target_column,
+            test_data=None,
+            OHE_columns=None,
+            high_cardinality_columns=None,
+            OHE_columns_cutoff=5,
+            train_size=0.7,
+            model_prior=None,
+            model_post=None
+            ):
         """Compares the ML efficacy of the prior data to the post data
         For a given `target_column`, this builds a ML model separately with
         `df_prior` and `df_post`, and compares the performance
@@ -479,23 +478,19 @@ class DataDriftDetector:
             List of columns to be one hot encoded, will be determined
             if not provided
         high_cardinality_columns: <list of str>
-            List of columns to be cat boost encoded, will be
+            List of columns to be count-encoded, will be
             determined if not provided
         OHE_columns_cutoff: <int>
             Number of unique labels in a column to determine OHE_columns &
-            high_cardinality_columns if not provided. 
-        random_state: <int>
-            Random state for the RandomizedSearchCV & the model fitting
+            high_cardinality_columns if not provided.
         train_size: <float>
             Proportion to split the df_post by, if test_data is not provided
-        cv: <int>
-            Number of cross validation folds to be used in the
-            RandomizedSearchCV
-        n_iter: <int>
-            Number of iterations for the RandomizedSearchCV
-        param_grid: <dictionary of parameters>
-            Dictionary of hyperparameter values to be iterated by
-            the RandomizedSearchCV
+        model_prior: <class>
+            A model with minimally a `fit` and `predict` methods used to train the prior
+            dataset. If none is provided, a default is used
+        model_post: <class>
+            A model with minimally a `fit` and `predict` methods used to train the post
+            dataset. If none is provided, a default is used
         Returns
         ----
         Returns a report of ML metrics between the prior model and the
@@ -512,16 +507,10 @@ class DataDriftDetector:
         assert isinstance(high_cardinality_columns, (list, type(None))),\
             "high_cardinality_columns should be of type list"
 
-
-        # TODO: - Allow choice of model?
-        #       - Allow choice of encoding for high cardinality cols?
-
         self.target_column = target_column
         self.train_size = train_size
-        self.random_state = random_state
-        self.cv = cv
-        self.n_iter = n_iter
-        self.param_grid = param_grid
+        self.model_prior = model_prior
+        self.model_post = model_post
 
         col_nunique = self.df_prior.nunique()
 
@@ -553,20 +542,71 @@ class DataDriftDetector:
         self._ml_data_prep()
 
         if target_column in self.categorical_columns:
-            self._build_classifier()
+            # set default model if not provided
+            if self.model_prior == None:
+                self.model_prior = RandomizedSearchCV(
+                    estimator=RandomForestClassifier(random_state=42),
+                    param_distributions={
+                        "n_estimators": [100, 200],
+                        "max_samples": [0.6, 0.8, 1],
+                        "max_depth": [3, 4, 5]
+                    },
+                    n_iter=10,
+                    cv=10,
+                    random_state=42
+                )
+            if self.model_post == None:
+                self.model_post = RandomizedSearchCV(
+                    estimator=RandomForestClassifier(random_state=42),
+                    param_distributions={
+                        "n_estimators": [100, 200],
+                        "max_samples": [0.6, 0.8, 1],
+                        "max_depth": [3, 4, 5]
+                    },
+                    n_iter=10,
+                    cv=10,
+                    random_state=42
+                )
+            # actually fit and evaluate model
+            self._fit_model()
             self._eval_classifier()
 
         elif target_column in self.numeric_columns:
-            self._build_regressor()
+            # set default model if not provided
+            if self.model_prior == None:
+                self.model_prior = RandomizedSearchCV(
+                    estimator=RandomForestRegressor(random_state=42),
+                    param_distributions={
+                        "n_estimators": [100, 200],
+                        "max_samples": [0.6, 0.8, 1],
+                        "max_depth": [3, 4, 5]
+                    },
+                    n_iter=10,
+                    cv=10,
+                    random_state=42
+                )
+            if self.model_post == None:
+                self.model_post = RandomizedSearchCV(
+                    estimator=RandomForestRegressor(random_state=42),
+                    param_distributions={
+                        "n_estimators": [100, 200],
+                        "max_samples": [0.6, 0.8, 1],
+                        "max_depth": [3, 4, 5]
+                    },
+                    n_iter=10,
+                    cv=10,
+                    random_state=42
+                )
+            # actually fit and evaluate model
+            self._fit_model()
             self._eval_regressor()
 
         return self.ml_report
 
 
     def _ml_data_prep(self):
-        """Prepares datasets for ML
-        This does one hot encoding, cat boost encoding, and train test
-        split (if necessary).
+        """Prepares datasets for ML. This does one hot encoding, count encoding
+        and train test split (if necessary)
         """
 
         df_post = self.df_post.copy()
@@ -599,16 +639,13 @@ class DataDriftDetector:
             test = test[cols].dropna(how='any')
             train_post = df_post
 
-        # determine columns for OHE & CatBoost
-        OHE_columns = [col for col in self.OHE_columns if
-                       col != self.target_column]
-        high_cardinality_columns = [col for col in self.high_cardinality_columns
-                                 if col != self.target_column]
-
-        if len(OHE_columns) > 0:
-            logger.info("One hot encoded columns: {}".format(", ".join(OHE_columns)))
-        if len(high_cardinality_columns) > 0:
-            logger.info("Cat boost encoded columns: {}".format(", ".join(high_cardinality_columns)))
+        # determine columns for OHE & High Cardinality
+        OHE_columns = [
+            col for col in self.OHE_columns if col != self.target_column
+        ]
+        high_cardinality_columns = [
+            col for col in self.high_cardinality_columns if col != self.target_column
+        ]
 
         # concat and then OHE to ensure columns match
         train_prior['source'] = "Train Prior"
@@ -616,45 +653,47 @@ class DataDriftDetector:
         train_post['source'] = "Train Post"
 
         df = pd.concat([train_prior, test, train_post])
-        df = pd.get_dummies(data=df, columns=OHE_columns)
+        if len(OHE_columns) > 0:
+            logger.info(f"One hot encoded columns: {OHE_columns}")
+            df = pd.get_dummies(data=df, columns=OHE_columns)
 
         train_prior = df[df.source == 'Train Prior'].drop('source', axis=1)
         test = df[df.source == 'Test'].drop('source', axis=1)
         train_post = df[df.source == 'Train Post'].drop('source', axis=1)
 
-        # CatBoostEncoder for high cardinality columns
         test_prior = test.copy()
         test_post = test.copy()
 
-        tf_prior = CatBoostEncoder(cols=high_cardinality_columns,
-                                   random_state=self.random_state)
-        tf_post = CatBoostEncoder(cols=high_cardinality_columns,
-                                  random_state=self.random_state)
+        if len(high_cardinality_columns) > 0:
+            logger.info(f"High cardinality columns: {high_cardinality_columns}")
+            # CountEncoder for high cardinality columns
+            tf_prior = CountEncoder(cols=high_cardinality_columns)
+            tf_post = CountEncoder(cols=high_cardinality_columns)
 
-        train_prior[high_cardinality_columns] = (
-            tf_prior.fit_transform(train_prior[high_cardinality_columns],
-                                   train_prior[self.target_column])
-        )
-        test_prior[high_cardinality_columns] = (
-            tf_prior.transform(test_prior[high_cardinality_columns],
-                               test_prior[self.target_column])
-        )
-        train_post[high_cardinality_columns] = (
-            tf_post.fit_transform(train_post[high_cardinality_columns],
-                                  train_post[self.target_column])
-        )
-        test_post[high_cardinality_columns] = (
-            tf_post.transform(test_post[high_cardinality_columns],
-                              test_post[self.target_column])
-        )
+            train_prior[high_cardinality_columns] = tf_prior.fit_transform(
+                train_prior[high_cardinality_columns],
+                train_prior[self.target_column]
+            )
+            test_prior[high_cardinality_columns] = tf_prior.transform(
+                test_prior[high_cardinality_columns],
+                test_prior[self.target_column]
+            )
+            train_post[high_cardinality_columns] = tf_post.fit_transform(
+                train_post[high_cardinality_columns],
+                train_post[self.target_column]
+            )
+            test_post[high_cardinality_columns] = tf_post.transform(
+                test_post[high_cardinality_columns],
+                test_post[self.target_column]
+            )
 
         X_train_prior = train_prior.drop(self.target_column, axis=1).astype(float)
-        y_train_prior = train_prior[self.target_column].astype(float)
+        y_train_prior = train_prior[self.target_column]
         X_test_prior = test_prior.drop(self.target_column, axis=1).astype(float)
-        y_test = test[self.target_column].astype(float)
+        y_test = test[self.target_column]
 
         X_train_post = train_post.drop(self.target_column, axis=1).astype(float)
-        y_train_post = train_post[self.target_column].astype(float)
+        y_train_post = train_post[self.target_column]
         X_test_post = test_post.drop(self.target_column, axis=1).astype(float)
 
         self.X_train_prior = X_train_prior
@@ -666,81 +705,15 @@ class DataDriftDetector:
         self.X_test_post = X_test_post
 
 
-    def _build_regressor(self):
+    def _fit_model(self):
+        """Fits the ML models
         """
-        Builds a random forest regressor with a RandomizedSearchCV
-        """
-
-        model_prior_ = RandomForestRegressor(random_state=self.random_state)
-        model_post_ = RandomForestRegressor(random_state=self.random_state)
-
-        model_prior = RandomizedSearchCV(model_prior_,
-                                         self.param_grid,
-                                         n_iter=self.n_iter,
-                                         cv=self.cv,
-                                         random_state=self.random_state)
-        model_post = RandomizedSearchCV(model_post_,
-                                        self.param_grid,
-                                        n_iter=self.n_iter,
-                                        cv=self.cv,
-                                        random_state=self.random_state)
-
-        model_prior.fit(self.X_train_prior, self.y_train_prior)
-        model_post.fit(self.X_train_post, self.y_train_post)
-        
-        msg = (
-            "A RandomForestRegressor with a RandomizedSearchCV was trained." +
-            "\nThe final model (trained with prior data) parameters are: " +
-            json.dumps(model_prior.best_params_) +
-            "\nThe final model (trained with post data) parameters are: " +
-            json.dumps(model_post.best_params_)
-        )
-
-        logger.info(msg)
-
-        self.model_prior = model_prior
-        self.model_post = model_post
-
-
-    def _build_classifier(self):
-        """
-        Build a random forest classifier with a RandomizedSearchCV
-        """
-
-        model_prior_ = RandomForestClassifier(random_state=self.random_state)
-        model_post_ = RandomForestClassifier(random_state=self.random_state)
-
-        model_prior = RandomizedSearchCV(model_prior_,
-                                        self.param_grid,
-                                        n_iter=self.n_iter,
-                                        cv=self.cv,
-                                        random_state=self.random_state)
-        model_post = RandomizedSearchCV(model_post_,
-                                         self.param_grid,
-                                         n_iter=self.n_iter,
-                                         cv=self.cv,
-                                         random_state=self.random_state)
-
-        model_prior.fit(self.X_train_prior, self.y_train_prior)
-        model_post.fit(self.X_train_post, self.y_train_post)
-        
-        msg = (
-            "A RandomForestClassifier with a RandomizedSearchCV was trained." +
-            "\nThe final model (trained with prior data) parameters are: " +
-            json.dumps(model_prior.best_params_) +
-            "\nThe final model (trained with post data) parameters are: " +
-            json.dumps(model_post.best_params_)
-        )
-
-        logger.info(msg)
-
-        self.model_prior = model_prior
-        self.model_post = model_post
+        self.model_prior.fit(self.X_train_prior, self.y_train_prior)
+        self.model_post.fit(self.X_train_post, self.y_train_post)
 
 
     def _eval_regressor(self):
-        """
-        Calculates the RMSE, MAE & R2 score of the prior and post model.
+        """Calculates the regression metrics of the prior and post model.
         Returns a pandas dataframe of the results.
         """
 
@@ -750,15 +723,21 @@ class DataDriftDetector:
         rmse_prior = self._rmse(self.y_test, y_pred_prior)
         mae_prior = mean_absolute_error(self.y_test, y_pred_prior)
         r2_prior = r2_score(self.y_test, y_pred_prior)
+        mape_prior = mean_absolute_percentage_error(self.y_test, y_pred_prior)
+        evs_prior = explained_variance_score(self.y_test, y_pred_prior)
 
         rmse_post = self._rmse(self.y_test, y_pred_post)
         mae_post = mean_absolute_error(self.y_test, y_pred_post)
         r2_post = r2_score(self.y_test, y_pred_post)
+        mape_post = mean_absolute_percentage_error(self.y_test, y_pred_post)
+        evs_post = explained_variance_score(self.y_test, y_pred_post)
 
         res = pd.DataFrame({
-            'RMSE': [rmse_prior, rmse_post],
-            'MAE': [mae_prior, mae_post],
-            'R2': [r2_prior, r2_post]
+            'root_mean_squared_error': [rmse_prior, rmse_post],
+            'mean_absolute_error': [mae_prior, mae_post],
+            'mean_absolute_percentage_error': [mape_prior, mape_post],
+            'explained_variance_score': [evs_prior, evs_post],
+            'r2_score': [r2_prior, r2_post]
             },
             index=['Prior', 'Post']
         )
@@ -767,18 +746,16 @@ class DataDriftDetector:
 
 
     def _eval_classifier(self):
-        """
-        Calculates the accuracy, precision, recall, F1 score & AUC of the
-        prior and post model.
+        """Calculates the classification metrics of the prior and post model.
         Returns a pandas dataframe of the result.
         """
 
-        y_pred_prior = self.model_prior.predict(self.X_test_prior)
-        y_pred_post = self.model_post.predict(self.X_test_post)
+        y_pred_prior_arr = self.model_prior.predict(self.X_test_prior)
+        y_pred_post_arr = self.model_post.predict(self.X_test_post)
 
-        y_test_ = pd.DataFrame(self.y_test)
-        y_pred_prior = pd.DataFrame(y_pred_prior, columns=y_test.columns)
-        y_pred_post = pd.DataFrame(y_pred_post, columns=y_test.columns)
+        y_test = pd.DataFrame(self.y_test)
+        y_pred_prior = pd.DataFrame(y_pred_prior_arr, columns=y_test.columns)
+        y_pred_post = pd.DataFrame(y_pred_post_arr, columns=y_test.columns)
 
         y_pred_prior['source'] = "prior"
         y_pred_post['source'] = "post"
@@ -797,20 +774,23 @@ class DataDriftDetector:
 
         res = pd.DataFrame([])
 
-        if (len(y_test[0]) == 2):
+        if len(y_test[0]) == 2:
             # for binary classification
             # only take position 1, assuming position 1 is the true label
             iters = [1]
-        else:
+        elif len(y_test[0]) > 2:
             # for multiclass classification
             iters = range(len(y_test[0]))
 
         for i in iters:
+            N = np.count_nonzero(y_test[:,i])
 
             precision_prior = precision_score(y_test[:,i], y_pred_prior[:,i])
             recall_prior = recall_score(y_test[:,i], y_pred_prior[:,i])
             acc_prior = accuracy_score(y_test[:,i], y_pred_prior[:,i])
             f1_prior = f1_score(y_test[:,i], y_pred_prior[:,i])
+            conf_mat_prior = confusion_matrix(y_test[:,i], y_pred_prior[:,i])
+            tn_pr, fp_pr, fn_pr, tp_pr = conf_mat_prior.ravel()
             try:
                 auc_prior = roc_auc_score(y_test[:,i], y_pred_prior[:,i])
             except ValueError:
@@ -820,27 +800,151 @@ class DataDriftDetector:
             recall_post = recall_score(y_test[:,i], y_pred_post[:,i])
             acc_post = accuracy_score(y_test[:,i], y_pred_post[:,i])
             f1_post = f1_score(y_test[:,i], y_pred_post[:,i])
+            conf_mat_post = confusion_matrix(y_test[:,i], y_pred_post[:,i])
+            tn_pt, fp_pt, fn_pt, tp_pt = conf_mat_post.ravel()
             try:
                 auc_post = roc_auc_score(y_test[:,i], y_pred_post[:,i])
             except ValueError:
                 auc_post = "NA"
 
-            multiindex = [(str(class_labels[i]), 'Prior'),
-                         (str(class_labels[i]), 'Post')]
+            multiindex = [
+                (str(class_labels[i]), 'Prior'), (str(class_labels[i]), 'Post')
+            ]
 
-            index = pd.MultiIndex.from_tuples(multiindex,
-                                              names=['Class', 'Data Type'])
+            index = pd.MultiIndex.from_tuples(
+                multiindex,
+                names=['Class', 'Data Type']
+            )
 
             score = pd.DataFrame({
-                'Accuracy': [acc_prior, acc_post],
-                'Precision': [precision_prior, precision_post],
-                'Recall': [recall_prior, recall_post],
-                'F1': [f1_prior, f1_post],
-                'AUC': [auc_prior, auc_post]
+                'accuracy': [acc_prior, acc_post],
+                'precision': [precision_prior, precision_post],
+                'recall': [recall_prior, recall_post],
+                'f1_score': [f1_prior, f1_post],
+                'roc_auc_score': [auc_prior, auc_post],
+                'TN': [tn_pr, tn_pt],
+                'FP': [fp_pr, fp_pt],
+                'FN': [fn_pr, fn_pt],
+                'TP': [tp_pr, tp_pt],
+                'N': [N, N]
                 },
                 index=index
             )
 
             res = pd.concat([res, score])
 
-        self.ml_report = res
+        # if multiclass, calculate macro, micro & weight averaging of scores
+        if len(y_test[0]) > 2:
+            N = len(y_test)
+            
+            # macro average
+            ind = "_MACRO_AVERAGE"
+            index = pd.MultiIndex.from_tuples(
+                [(ind, 'Prior'), (ind, 'Post')],
+                names=['Class', 'Data Type']
+            )
+            prior_avg = res[res.index.get_level_values(1) == "Prior"].mean()
+            post_avg = res[res.index.get_level_values(1) == "Post"].mean()
+            prior_sum = res[res.index.get_level_values(1) == "Prior"].sum()
+            post_sum = res[res.index.get_level_values(1) == "Post"].sum()
+            macro_roc_auc_ovr_prior = roc_auc_score(
+                y_test,
+                y_pred_prior,
+                multi_class="ovr",
+                average="macro",
+            )
+            macro_roc_auc_ovr_post = roc_auc_score(
+                y_test,
+                y_pred_post,
+                multi_class="ovr",
+                average="macro",
+            )
+
+            macro_avg_res = pd.DataFrame({
+                'accuracy': [prior_avg["accuracy"], post_avg["accuracy"]],
+                'precision': [prior_avg["precision"], post_avg["precision"]],
+                'recall': [prior_avg["recall"], post_avg["recall"]],
+                'f1_score': [prior_avg["f1_score"], post_avg["f1_score"]],
+                'roc_auc_score': [macro_roc_auc_ovr_prior, macro_roc_auc_ovr_post],
+                'TN': [prior_sum["TN"], post_sum["TN"]],
+                'FP': [prior_sum["FP"], post_sum["FP"]],
+                'FN': [prior_sum["FN"], post_sum["FN"]],
+                'TP': [prior_sum["TP"], post_sum["TP"]],
+                'N': [N, N]
+                },
+                index=index
+            )
+
+            # micro average
+            ind = "_MICRO_AVERAGE"
+            index = pd.MultiIndex.from_tuples(
+                [(ind, 'Prior'), (ind, 'Post')],
+                names=['Class', 'Data Type']
+            )
+            recall_prior = prior_sum["TP"] / (prior_sum["TP"] + prior_sum["FN"])
+            recall_post = post_sum["TP"] / (post_sum["TP"] + post_sum["FN"])
+            precision_prior = prior_sum["TP"] / (prior_sum["TP"] + prior_sum["FP"])
+            precision_post = post_sum["TP"] / (post_sum["TP"] + post_sum["FP"])
+            f1_prior = (
+                (2 * precision_prior * recall_prior) / (precision_prior + recall_prior)
+            )
+            f1_post = (
+                (2 * precision_post * recall_post) / (precision_post + recall_post)
+            )
+            micro_roc_auc_ovr_prior = roc_auc_score(
+                y_test,
+                y_pred_prior,
+                multi_class="ovr",
+                average="micro",
+            )
+            micro_roc_auc_ovr_post = roc_auc_score(
+                y_test,
+                y_pred_post,
+                multi_class="ovr",
+                average="micro",
+            )
+
+            micro_avg_res = pd.DataFrame({
+                'accuracy': [recall_prior, recall_post],
+                'precision': [precision_prior, precision_post],
+                'recall': [recall_prior, recall_post],
+                'f1_score': [f1_prior, f1_post],
+                'roc_auc_score': [micro_roc_auc_ovr_prior, micro_roc_auc_ovr_post],
+                'TN': [prior_sum["TN"], post_sum["TN"]],
+                'FP': [prior_sum["FP"], post_sum["FP"]],
+                'FN': [prior_sum["FN"], post_sum["FN"]],
+                'TP': [prior_sum["TP"], post_sum["TP"]],
+                'N': [N, N]
+                },
+                index=index
+            )
+
+            # weighted average
+            ind = "_WEIGHTED_AVERAGE"
+            index = pd.MultiIndex.from_tuples(
+                [(ind, 'Prior'), (ind, 'Post')],
+                names=['Class', 'Data Type']
+            )
+            cols = [c for c in res.columns if c != "N"]
+            _res = res[cols].multiply(res["N"], axis=0)
+
+            prior_avg = _res[_res.index.get_level_values(1) == "Prior"].sum() / N
+            post_avg = _res[_res.index.get_level_values(1) == "Post"].sum() / N
+
+            weighted_avg_res = pd.DataFrame({
+                'accuracy': [prior_avg["accuracy"], post_avg["accuracy"]],
+                'precision': [prior_avg["precision"], post_avg["precision"]],
+                'recall': [prior_avg["recall"], post_avg["recall"]],
+                'f1_score': [prior_avg["f1_score"], post_avg["f1_score"]],
+                'roc_auc_score': [prior_avg["roc_auc_score"], post_avg["roc_auc_score"]],
+                'TN': [prior_sum["TN"], post_sum["TN"]],
+                'FP': [prior_sum["FP"], post_sum["FP"]],
+                'FN': [prior_sum["FN"], post_sum["FN"]],
+                'TP': [prior_sum["TP"], post_sum["TP"]],
+                'N': [N, N]
+                },
+                index=index
+            )
+            res = pd.concat([res, micro_avg_res, macro_avg_res, weighted_avg_res])
+
+        self.ml_report = res
\ No newline at end of file
